////////////////////////////////////////////////////////////
//   ____  ____ 
//  /   /\/   / 
// /___/  \  /    Vendor: Xilinx 
// \   \   \/     Version : 7.1i
//  \   \         Application : ISE
//  /   /         Filename : read_ps2_a.v
// /___/   /\     Timestamp : 09/20/2004 11:12:50
// \   \  /  \ 
//  \___\/\___\ 
//
// This module interprets the scan code from the keyboard  
// and outputs the corresponding signals. 
//
// Revison: 24 March 2006 by Anton
// Enhanced: 03, 05, 11 April 2006 by Harvey to include
// alphabet, numbers, backspace, (select arrows or numeric keypad)
// Simplified, made easier to use: 27 July 2006 by Anton
// Renamed from read_ps2_AN.v to read_ps2_a.v on 28, 29 July by Harvey
// Rev 26 Feb 2007 Anton  added stopkey: pulses hi when key has been released
// 26 Sep 07 Anton made interface same for verilog and vhdl versions of ps2
// renamed as kbd_anton2.vhd
// 27 Sep 07 HW deleted "wire data_ready" since "inout data_ready" exists.
// added comments, re-arrainged some code
/////////////////////////////////////////////////

module kbd_anton2 (Clk, PS2_Clk, PS2_Data, data_ready, ps2_code, number, stopkey);

    input Clk;              // main clock
    input PS2_Clk;          // keyboard clock
    input PS2_Data;         // keyboard data
    inout data_ready;       // indicates when scancode is available
    inout [7:0] ps2_code;   // scancode from keyboard
    output [3:0] number;    // hexidecimal numbers 0-9, A-F
    output stopkey;         // pulses hi when key has been released

    wire trigger;
// above wire signals originate with a component module
// below reg signals originate with this module
    reg read, state, stopkey;
    reg [3:0] number;

	PS2_CTRL ps2_ctrl ( 
	.Clk(Clk),               // input fast clock
	.Reset(1'b0),            // input reset (active high)
	.PS2_Clk(PS2_Clk),       // input clock from PS2 port (~20-30 KHz)
	.PS2_Data(PS2_Data),     // input data from PS2 port
	.Scan_Code(ps2_code),    // output Scan code representing keyboard key received
	.scan_ready(data_ready), // output indicating that a new Scan Code has been received
	.trigger(trigger),       // output Trigger resets the read signal
	.DoRead(read)            // input signifying ready to receive another scan code
	);

always @(posedge Clk)

  if ((data_ready) | (!read))	// New key data has arrived
    begin
      case (state)
        1'b0 : // state is 1'b0
          begin
            if (!stopkey)	// new data is not releasing a key
              begin
                state <= 1'b1;
                case (ps2_code)
                  8'h45: number <= 4'h0; // top   0 key
                  8'h70: number <= 4'h0; // right 0 key
                  8'h16: number <= 4'h1; // top   1 key
                  8'h69: number <= 4'h1; // right 1 key
                  8'h1e: number <= 4'h2; // top   2 key
                  8'h72: number <= 4'h2; // right 2 key
                  8'h26: number <= 4'h3; // top   3 key
                  8'h7a: number <= 4'h3; // right 3 key
                  8'h25: number <= 4'h4; // top   4 key
                  8'h6b: number <= 4'h4; // right 4 key
                  8'h2e: number <= 4'h5; // top   5 key
                  8'h73: number <= 4'h5; // right 5 key
                  8'h36: number <= 4'h6; // top   6 key
                  8'h74: number <= 4'h6; // right 6 key
                  8'h3d: number <= 4'h7; // top   7 key
                  8'h6c: number <= 4'h7; // right 7 key
                  8'h3e: number <= 4'h8; // top   8 key
                  8'h75: number <= 4'h8; // right 8 key
                  8'h46: number <= 4'h9; // top   9 key
                  8'h7d: number <= 4'h9; // right 9 key
                  // Since hexadecimal numbers above 9
                  // are also alpha characters, then
                  // letters A through F are also
                  // assigned as numbers.
                  // letters G through Z are not included
                  8'h1c: number <= 4'ha;
                  8'h32: number <= 4'hb;
                  8'h21: number <= 4'hc;
                  8'h23: number <= 4'hd;
                  8'h24: number <= 4'he;
                  8'h2b: number <= 4'hf;
                  8'hf0: stopkey <= 1'b1; // set stopkey bit, A key has been released
                  default: number <= 4'h0; // 0
                endcase  		   	 // end of case (ps2_code) statement
              end // end begin (!stopkey)
            else   // must be (stopkey)   // new data is telling which key was just released
              begin
                case (ps2_code)
                  8'hf0: stopkey <= 1'b1;	 // Check next ps2 entry for stop											 
                  default: stopkey <= 1'b0;
                endcase 			 // end of case (ps2_code) statement
                state <= 1'b1;
              end // end begin else (stopkey) 
            read = 1'b0;
          end // end begin state is 1'b0
                               
        1'b1 : // state is 1'b1
          begin
            if (!trigger)
              begin 
                read = 1'b1;  // resets read bit to enable PS2_Ctrl input
                state <= 1'b0;
              end // end begin (!trigger)
          end // end begin state is 1'b1
        endcase
    end // end begin if ((data_ready) | (!read))

endmodule

